<?php
// $Id: messaging_tools.admin.inc,v 1.1.2.2 2010/05/15 16:42:59 jareyero Exp $
/**
 * @file
 * Messaging Framework - Queue Admin UI
 */

/**
 * Queue options form
 */
function messaging_tools_queue_settings($form_state) {
  $form['help']['#value'] = messaging_tools_queue_help();
  $form['messaging_queue_process_cron'] = array(
    '#type' => 'radios',
    '#title' => t('Process on cron'),
    '#default_value' => variable_get('messaging_queue_process_cron', 1),
    '#options' => array(t('Disabled'), t('Enabled')),
    '#description' => t('If disabled you must use code elsewhere to actually send your queued messages such as a <a href="http://drupal.org/project/drush">drush</a> script.'),
  );
  // Logging and expiration settings  
  $period = array(0 => t('Disabled')) + drupal_map_assoc(array(3600, 10800, 21600, 32400, 43200, 86400, 172800, 259200, 604800, 1209600, 2419200, 4838400, 9676800), 'format_interval');
  $form['messaging_log'] = array(
    '#title' => t('Logging'),
    '#type' => 'select',
    '#options' => $period,
    '#default_value' => variable_get('messaging_log', 0),
    '#description' => t('If enabled all messages for the <a href="@messaging-methods">selected methods</a> will be logged and kept for the specified time after they\'re sent.', array('@messaging-methods' => url('admin/messaging/settings/method'))),
  );
  $form['messaging_queue_expire'] = array(
    '#title' => t('Expiration'),
    '#type' => 'select',
    '#options' => $period,
    '#default_value' => variable_get('messaging_queue_expire', 0),
    '#description' => t('If enabled all queued messages will be deleted after the selected time, even when they haven\'t been sent yet.'),
    '#disabled' => !messaging_store()->can_expire(),
  );
  /*
  $form['messaging_queue_pending_limit'] = array(
    '#title' => t('Limit pending messages'),
    '#type' => 'textfield',
    '#size' => 10,
    '#default_value' => variable_get('messaging_queue_pending_limit', 0),
    '#description' => t('Discard older messages when the queue exceeds this limit. Some messages may be lost but this will keep your processing under control. Set 0 (zero) for no limit.'),
  );
  */
  $limit = variable_get('messaging_process_limit', array('message' => 0, 'percent' => MESSAGING_DEFAULT_CRON_PERCENT, 'time' => 0));
  $form['messaging_process_limit'] = array(
    '#type' => 'fieldset',
    '#title' => t('Limits for queue processing'),
    '#tree' => TRUE,
    '#collapsible' => TRUE, '#collapsed' => !array_filter($limit),
    '#description' => t('These are the limits for each cron run on queue processing. The process will stop when it first meets any of them. Set to 0 for no limit.'),
  );
  $form['messaging_process_limit']['message'] = array(
    '#title' => t('Number of messages sent'),
    '#type' => 'textfield',
    '#size' => 10,
    '#default_value' => $limit['message'],
  );
  $form['messaging_process_limit']['time'] = array(
    '#title' => t('Time (seconds)'),
    '#type' => 'textfield',
    '#size' => 10,
    '#default_value' => $limit['time'],
  );
  $form['messaging_process_limit']['percent'] = array(
    '#title' => t('Time (% of cron time)'),
    '#type' => 'textfield',
    '#size' => 10,
    '#default_value' => $limit['percent'],
    '#description' => t('Maximum percentage of cron time the process may use.'),
  );
  // The Queue class can provide some more settings
  if ($more_settings = messaging_store()->admin_settings()) {
    $form = array_merge($form, $more_settings);
  }

  return system_settings_form($form);  
}

/**
 * Queue status page
 */
function messaging_tools_queue_status() { 
  $output = messaging_tools_queue_help();
  // Queued messages
  $queued = messaging_store()->queue_count();
  
  // Compile data from db
  $status = array('queue' => 0, 'error' => 0, 'log' => 0, 'other' => 0);
  //$result = db_query("SELECT method, queue, cron, error, log, count(mqid) AS total FROM {messaging_store} GROUP BY method, queue, cron, error, log");
  $summary = messaging_store()->get_status(array('method', 'queue', 'cron', 'error', 'log'));
  $messages = array();
  if ($summary) {
    foreach ($summary as $data) {
      if (is_array($data)) {
        $method = $data['method'];
        if (!isset($methods[$method])) {
          $methods[$method] = array('queue' => 0, 'error' => 0, 'log' => 0, 'other' => 0);
        }
        $value = (int)$data['total'];
        if ($data['error']) {
          $field = 'error';
        }
        elseif ($data['queue']) {
          $field = 'queue';
        }
        elseif ($data['log']) {
          $field = 'log';
        }
        else {
          $field = 'other';
        }
        $methods[$method][$field] += $value;
        $status[$field] += $value;
      }
      else {
        // Not an array it must be an status message
        $messages[] = $data;
      }
    }
  }
  // Build table if we've got any data
  if ($methods) {
    $send_methods = messaging_method_list();
    $header = array(t('Method'), t('Queued'), t('Errors'), t('Logs'), t('Unknown'), t('total'));
    foreach ($methods as $method => $data) {
      $rows[] = array(
        isset($send_methods[$method]) ? '<strong>' . $send_methods[$method] . '</strong>' : $method . ' ' . t('disabled'),
        $data['queue'], $data['error'], $data['log'], $data['other'], array_sum($data)
      );
    }
    $status['total'] = array_sum($status);
    $rows[] = array(t('Total'), $status['queue'], $status['error'], $status['log'], $status['other'], $status['total']);
    $content = theme('table', $header, $rows);

    $output .= drupal_get_form('messaging_tools_queue_operations_form', $queued, $status);
  }
  else {
    $content = t('No data available.');
  }
  if ($messages) {
    $content .= theme('item_list', $messages);
  }
  $output .= theme('box', t('Queued messages'), $queued ? '<strong>' . $queued . '</strong> ' . t('(All methods)') : t('No messages in queue.'));  
  $output .= theme('box', t('Stored messages'), $content);
  return $output;
}

/**
 * Queue operations form
 */
function messaging_tools_queue_operations_form($form_state, $queued, $status) {
  $form = $help = array();
  if ($queued) {
    $operations['queue'] = array('#type' => 'submit', '#value' => t('Process queue'));
  }
  if ($status['total']) {
    $operations['clean'] = array('#type' => 'submit', '#value' => t('Clean up'));
    $help[] = t('The <i>Clean up</i> operation will remove expired logs and messages (Or all logs if no expiration set).');
    $operations['delete'] = array(
      '#type' => 'submit', '#value' => t('Delete all'),
      '#prefix' => '<div style="float:right;">',
      '#suffix' => '</div>',
    );
  }
  if (!empty($operations)) {
    // Pass current max row id, so any operation doesn't affect new rows being produced.
    $form['mqid'] = array(
      '#type' => 'value',
      '#value' => db_result(db_query("SELECT MAX(mqid) FROM {messaging_store}")),
    );
    $form['operations'] = $operations += array('#title' => t('Operations'), '#type' => t('fieldset'));
    if ($help) {
      $form['help']['#value'] = theme('item_list', $help);
    }
  }
  return $form;
}

/**
 * Submit queue operations form
 */
function messaging_tools_queue_operations_form_submit($form, $form_state) {
  $mqid = $form_state['values']['mqid'];
  $deleted = $processed = $errors = 0;

  switch ($form_state['values']['op']) {
    case t('Process queue'):
      // If cron is running, we better let the queue alone
      if (variable_get('messaging_queue_process_cron', TRUE) && variable_get('cron_semaphore', FALSE)) {
        drupal_set_message(t('We cannot manually process messages while cron is running. Try again later and/or disable queue processing on cron.'), 'warning');
      }
      else {
        $limit = variable_get('messaging_process_limit', array('message' => 0, 'time' => 0, 'percent' => 0));
        $results = messaging_store()->queue_process($limit);
        if ($results) {
          $processed = count($results);
          $success = count(array_filter($results));
          $errors = $processed - $success;
        }
      }
      break;
 
    case t('Clean up');
      //$deleted = messaging_store('queue_clean', TRUE);
      $deleted = messaging_store()->queue_clean(TRUE);
      break;

    case t('Delete all');
      $deleted = messaging_store()->delete_all();
      break;

  }
  if ($processed) {
    drupal_set_message(t('Processed @count messages from queue.', array('@count' => $processed)));
    if ($errors) {
      drupal_set_message(t('Found @count errors during queue processing.', array('@count' => $errors)), 'warning');
    }
  }
  if (!empty($deleted)) {
    drupal_set_message(t('Deleted @count queued messages and logs.', array('@count' => $deleted)));
  }
}

/**
 * Print out help from messaging store
 */
function messaging_tools_queue_help() {
  $help = messaging_store()->admin_help();
  $name = $help['name'];
  unset($help['name']);
  return theme('box', $name, theme('item_list', $help));
}

