<?php
// $Id: messaging_tools.drush.inc,v 1.1.2.1 2010/05/17 07:54:04 jareyero Exp $

/**
 * @file
 *   Drush command to process notifications queue.
 *   
 * Process the queued notifications as a stand alone shell command instead of using cron
 *  
 *  drush --uri=http://www.example.com notifications process [rows] [messages]
 *  
 * Notes: 
 * - To properly generate links the base url must be set. You may need to set it manually in settings.php
 * - Before using this you will want to disable cron for Notifications, there's an option for that on Admin pages.
 * - The limits for queue processing (See Admin pages) still apply, though the row limit can be overriden with [rowlimit]
 * - The autoload hack is needed by autoload module, http://drupal.org/node/547736
 */

/**
 * Implementation of hook_drush_command().
 * 
 * @see drush_parse_command() for a list of recognized keys.
 *
 * @return
 *   An associative array describing your command(s).
 */
function messaging_tools_drush_command() {
  $items['messaging process'] = array(
    'callback' => 'messaging_drush_queue_process',
    'description' => "Send out queued messages.",
    'arguments' => array(
      'messages' => 'Optional maximum number of messages to send. Zero for no limit.',
    ),
  );
  return $items;
}

/**
 * Implementation of hook_drush_help().
 */
function messaging_tools_drush_help($section) {
  switch ($section) {
    case 'drush:messaging process':
      return dt("Send out queued messages.");
  }
}


/**
 * Drush command callback, process queue
 * 
 * @param $rows
 *   Maximum number of rows to process
 */
function messaging_drush_queue_process($messages = NULL) {
  // We need to run our drush hack included with messaging.
  messaging_include('drush_hack.inc');

  // Start timer and prepare parameters
  timer_start('messaging_process');
  $limit = array();
  if (isset($messages)) {
    $limit['message'] = (int)$messages;
  }
  $queue = messaging_store();
  $result = $queue->queue_process($limit);
  $queue->queue_expire_messages();
  $queue->queue_expire_logs();
  // Calculate results
  $timer = timer_stop('messaging_process');
  $time = $timer['time'];
  $processed = count($result);
  $success = count(array_filter($result));
  $failed = $processed - $success;
  drush_print(dt("Processed @processed messages: sent @success, failed @failed in @time ms.", array('@processed' => $processed, '@success' => $success, '@failed' => $failed, '@time' => $time))); 
}

